%{
/*
   rtf-ascii [-lracC] [files...]

   Convert RTF to plaintext ASCII.  Input may be piped standard input,
   or one or more "rtf" files or "rtfd" directory wrappers (or ".attach"
   wrappers used by Mail).

   If an input stream is not RTF it is passed through unchanged.
   If the input RTF contains non-ascii characters, an attempt is made
   to print these somehow (e.g., an umlauted "u" is transcribed as "u:").
   With no options, an RTF file is converted to ASCII that is appropriate
   for most applications.

   Options are:
      -l "..."    use "..." as a left-bracket character
                  when printing attachment labels. ("/*" by default).
      -r "..."    use "..." as a left-bracket character
                  when printing attachment labels.
      -a "..."    don't print attachment labels.
      -c "..."    don't transcribe non-ascii characters.
      -C "..."    filter appropriate for the C language (-a -c).

   Notes:

   The following Lex grammar is for Microsoft RTF (Rich Text Format),
   rewritten (from my original rtfa program) with numerous modifications
   to accommodate NeXTSTEP 3.0 .rtfd, .attach, links, markers,
   and for the rtf-tolerant C preprocessor.  (Dec 1992).

   Passes plaintext ascii through untouched. (Mar 1993)

   NB: This will NOT work with Gnu "flex" (which does not permit the
   input() macro to be redfined).

   M. J. Hawley
   MIT Media Laboratory
   20 Ames Street
   Cambridge, MA 02139
   mike@media-lab.mit.edu
*/


/*
 * Now, pay attention!  Nothing up my sleeve...
 */
#define Case    break; case
#define Default break; default
char *_arg, *_argp, *av0; /* use by 'for_each_argument */
#define argument  (_arg=(*_argp?_argp:av[++i==ac?--i: i]),_argp+=strlen(_argp),_arg)
#define for_each_argument av0=av[0]; for (i=1;i<ac && *av[i]=='-';i++)\
                        for (_argp = &av[i][1]; *_argp;)\
                                switch(*_argp++)
int p;				/* used only for attachments */
int PrintNonAscii = 1;		/* cvt non-ascii to something printable */
int PrintAttachments = 1;	/* print a note about an attached item */
char *LeftBracket = "/*",	/* for output of attachment names */
     *RightBracket = "*/";

Getc(f) FILE *f; {  /* front end to convert non-ascii characters */
    char c;
    int x;
    static char s[8], *sp = (char *)0;
    static char *S[] = {0,"A`","A'","A^","A~","A:","A*","C,",
                   "E`","E'","E^","E:","I`","I'","I^","I:",
                   "-D","N~","O`","O'","O^","O~","O:","U`",
                   "U'","U^","U:","Y'","b","u","x","/",
                   "(c)","!","c","L","/","Y","f","S",
                   "*","'","``","<<","<",">","fi","fl",
                   "(R)","-","+","++",".","|","P","*",
                   ",",",,","''",">>","...","%.","-","?",
                   "1","`","'","^","~","-","u",".",
                   ":","2",".",",","3","\"",",","v",
                   "--","+-","1/4","1/2","3/4","a`","a'","a^",
                   "a~","a:","a*","c,","e`","e'","e^","e:",
                   "i`","AE","i'","a","i^","i:","d","n~",
                   "L/","O/","OE","o","o`","o'","o^","o~",
                   "o:","ae","u`","u'","u^","i","u:","y'",
                   "|","o","oe","B","b","y:"};
    if (sp){
        c = *sp++;
        if (!*sp) sp = (char *)0;
        return c;
    }
    x = getc(f);
    if (!PrintNonAscii || x <= 0177) return (char)x;
    if (x > 0200 && x < 0376 && x != 0254) strcpy(sp=s,S[x-0200]);
    if (!s) return (char)x;
    return Getc(f);
}

#undef input
#define input() (((yytchar=yysptr>yysbuf?U(*--yysptr):Getc(yyin))==10?(yylineno++,yytchar):yytchar)==EOF?0:yytchar)
%}

%e 2000
%p 5000
%o 2000
%n 1000
%a 2500

num	[-]?[0-9]+\ ?
optnum	([-]?[0-9]+)?\ ?
optsemi	\;?
semi	([^;]+;)
filename	[ /a-zA-Z0-9._-]
%%
\\[{}\\\n]	putchar(yytext[1]);

[{}\n]		;

\\(ansi|mac|pc|pca|defformat|fonttbl|pard|pict|brdr([stblr]|db|th|sh|dot|hair|bar|btw)|macpict|picscaled|footnote|(header|footer)[lrf]|ch(ftn|atn)|annotation|atnid|title|subject|author|operator|keywords|comment|doccomm|creatim|(rev|prin|bup)tim|fld(dirty|edit|lock|priv|inst|rslt)|[bi]xe|tc|bkmk(start|end)|facingp|widowctrl|hyphhotz|ftn(sep|sepc|cn|tj|bj|restart)|end(notes|doc)|landscape|fracwidth|nextfile|template|makebackup|defformat|revisions|margmirror|sectd|sbk(none|col|page|even|odd)|pgn(cont|restart|dec|[ul]c(rm|ltr))|line(restart|ppage|cont|betcol)|box|titlepg|endnhere|q[lrjc]|verta(lt|l|c|j)|intbl|keep(n?)|sbys|pagebb|noline|tq([rc]|dec)|tb|tl(dot|hyph|ul|th)|posx([clor]|{num})|posy(il|[tcb]|{num})|p([vh](mrg|pg)|col)|plain|clbrdr[btlr]|tr(owd|q[lrc]|(gaph|rh|left{num}))|clm(gf|rg)|ulnone|revised|ch(date|time|pgn|ftn|atn|ftnsep)|cell|row|par|sect|page|column|line|tab|[tr]xe[ ][a-zA-Z0-9]+|[\|~\-_:\*]|'[0-9a-fA-F][0-9a-fA-F])	;

\\(rtf|gray|def(f|tab)|sbasedon|snext|c[fb]|fc|wmetafile|wbitmap|pic(w|h|wgoal|hgoal|scalex|scaley|crop[tblr])|wbm(bitspixel|planes|widthbytes)|bin|version|vern|edmins|yr|mo|dy|hr|min|nof(pages|words|chars)|id|paper[wh]|tc[fl]|marg[lrtb]|gutter|deftab|(ftn|pgn|line)start|rev(prop|bar)|pgn(starts|x|y)|(header|footer)y|s|s[bal]|line(mod|x|starts)|col(s|sx)|[flr]i|tx|brsp|absw|dxfrtext|cellx|f(s?)|expnd|up|dn){num}	;

\\f(nil|roman|swiss|modern|script|decor|tech){semi}	;
\\((red|green|blue){num}|colortbl){optsemi}		;
\\([bi]|strike|outl|shad|(s?)caps|v|ul(w?|db)){optnum}	;

\\(NeXTHelpMarker|markername|linkMarkername){semi}	;
\\NeXTHelpLink{num}[ ]	;
\\linkFilename[ ]{filename}+;		include("link",index(yytext,' '));
\\attachment{num}[ ]{filename}+		include("attach",index(yytext, ' '));

\\NeXTGraphic{num}[ ]{filename}+[ ]\\width{num}[ ]\\height{num} {
		    if (p=index(yytext, ' ')){
                        char *x = (char *)index(p,'\\');
			if (x) *x = '\0';
			include("image",p);
                    }
		}
%%

#include "rtfopen.c"

FILE *
Open(s) char *s; {
    char t[1024]="";
    FILE *f;
    if (isDirectory(s)){
        sprintf(t,"%s/%s",s,"TXT.rtf");
        if (access(t,0)) sprintf(t,"%s/%s",s,"index.rtf");
    } else
        strcpy(t,s);
    f = fopen(t,"r");
    was_rtf = f? isrtf(fileno(f)) : 0;
    return f;
}

char *
striptail(s) char *s; { /* strip trailing ' '|';' */
    char *t = s+strlen(s)-1;
    while (t>s && (*t == ' ' || *t == ';')) *t-- = '\0';
    while (*s == ' ' || *s == '\t') ++s;
    return s;
}

include(label,file) char *label, *file; {
    if (file && PrintAttachments)
        printf("%s%s:%s%s",LeftBracket,label,striptail(file),RightBracket);
}

cat(){
    char c;
    while ((c = getc(yyin)) != EOF)
        putchar(c);
}

e(a,b,c,d,f) char *a; int b, c, d, f; {
    fprintf(stderr,a,b,c,d,f); fprintf(stderr,"\n");
}

use(){
    e("use: %s [-a] [-C] [-c] [-l ...] [-r ...] [files...]",av0);
    e("convert RTF files to ascii; options:");
    e("    -a    -- don't print notes for attachments");
    e("    -C    -- do the right thing for the C language");
    e("    -c    -- don't convert non-ascii characters");
    e("    -l ...-- left bracket symbol to use (%s)",LeftBracket);
    e("    -r ...-- right bracket symbol to use (%s)",RightBracket);
    exit();
}

main(ac,av) char *av[]; {
    int i;
    for_each_argument {
    case 'l': LeftBracket = argument;
    Case 'r': RightBracket = argument;
    Case 'a': PrintAttachments = 0;
    Case 'c': PrintNonAscii = 0;
    Case 'C': PrintAttachments = PrintNonAscii = 0;
    Default : use();
    }
    if (i<ac){
        while (i<ac)
            if (yyin = Open(av[i++])){
                if (was_rtf) yylex();
                else cat();
                fclose(yyin);
            }
    } else {
	char s[6];
        yyin = stdin;
        for(i=0;i<5;i++) s[i]=input();
        for(i=0;i<5;i++) unput(s[4-i]); s[5] = '\0';
	was_rtf = !strncmp(s,"{\\rtf",5);
	if (was_rtf) yylex();
	else printf("%s",s),cat();
    }
    if (was_rtf) printf("\n");
}
